/**
 * Created by gaurav.pandvia on 2/10/17.
 */
import { TourStepTemplateService } from './services/tour-step-template.service';
import { Directive, ElementRef, Input, NgZone, ViewContainerRef, EventEmitter, Output, } from '@angular/core';
import { BmTourService } from "./services/bm-tour.service";
import { TourContentService } from "./services/tour-content.service";
var BmTourAnchorDirective = (function () {
    function BmTourAnchorDirective(tourService, tourStepTemplate, _elementRef, viewContainerRef, ngZone, tourContentService) {
        // this.ngP =  new NgbPopover(_elementRef, _renderer, injector, componentFactoryResolver, viewContainerRef, config, ngZone);
        this.tourService = tourService;
        this.tourStepTemplate = tourStepTemplate;
        this.viewContainerRef = viewContainerRef;
        this.ngZone = ngZone;
        this.tourContentService = tourContentService;
        this.placement = "top";
        this.alignment = "left";
        this.spacing = 5;
        this.showCaret = true;
        this.showCursor = false;
        // private mouseLeaveContentEvent: any;
        // private mouseEnterContentEvent: any;
        // private documentClickEvent: any;
        this.show = new EventEmitter();
        this.hide = new EventEmitter();
        this.element = _elementRef;
    }
    BmTourAnchorDirective.prototype.ngOnInit = function () {
        this.tourService.register(this.bmTourAnchor, this);
    };
    BmTourAnchorDirective.prototype.showTourStep = function (step) {
        this.stepContext = step;
        this.popoverTemplate = this.tourStepTemplate.template;
        this.showCursor = step.showCursor;
        if (step.title)
            this.popoverTitle = step.title;
        if (step.alignment)
            this.alignment = step.alignment;
        if (step.showCaret)
            this.showCaret = step.showCaret;
        if (step.spacing)
            this.spacing = step.spacing;
        this.container = 'body';
        switch (step.placement) {
            case 'above':
            case 'top':
                this.placement = 'top';
                break;
            case 'below':
            case 'bottom':
                this.placement = 'bottom';
                break;
            case 'right':
            case 'after':
                this.placement = 'right';
                break;
            case 'left':
            case 'before':
                this.placement = 'left';
                break;
            default:
                this.placement = 'top';
        }
        var bindingOptions = this.createBoundOptions();
        this.openTourStep();
        if (!step.preventScrolling) {
        }
    };
    BmTourAnchorDirective.prototype.hideTourStep = function () {
        var _this = this;
        if (!this.component)
            return;
        var destroyFn = function () {
            _this.hide.emit(true);
            // destroy component
            _this.tourContentService.destroy(_this.component);
            _this.component = undefined;
        };
        clearTimeout(this.timeout);
        // if(!immediate) {
        //   this.timeout = setTimeout(destroyFn, this.tooltipHideTimeout);
        // } else {
        destroyFn();
        // }
    };
    BmTourAnchorDirective.prototype.createBoundOptions = function () {
        return {
            title: this.popoverTitle,
            template: this.popoverTemplate,
            viewContainerRef: this.viewContainerRef,
            host: this.viewContainerRef.element,
            placement: this.placement,
            alignment: this.alignment,
            showCaret: this.showCaret,
            showCursor: this.showCursor,
            cssClass: "myClass",
            spacing: this.spacing,
            context: this.stepContext
        };
    };
    BmTourAnchorDirective.prototype.openTourStep = function () {
        var _this = this;
        // if (this.component || this.tooltipDisabled) return;
        var immediate = true;
        // const time = immediate ? 0 : this.stepContext.popoverShowTimeout;
        var time = 0;
        // ngUpgrade bug
        // https://github.com/angular/angular/issues/12318
        this.ngZone.run(function () {
            clearTimeout(_this.timeout);
            _this.timeout = setTimeout(function () {
                _this.tourContentService.destroyAll();
                var options = _this.createBoundOptions();
                _this.component = _this.tourContentService.create(options);
                // add a tiny timeout to avoid event re-triggers
                // setTimeout(() => {
                //   this.addHideListeners(this.component.instance.element.nativeElement);
                // }, 10);
                _this.show.emit(true);
            }, time);
        });
    };
    BmTourAnchorDirective.prototype.ngOnDestroy = function () {
        if (this.tourService.status == 0) {
            this.tourService.unregister(this.bmTourAnchor);
        }
        this.hideTourStep();
    };
    return BmTourAnchorDirective;
}());
export { BmTourAnchorDirective };
BmTourAnchorDirective.decorators = [
    { type: Directive, args: [{
                selector: '[bmTourAnchor]',
            },] },
];
/** @nocollapse */
BmTourAnchorDirective.ctorParameters = function () { return [
    { type: BmTourService, },
    { type: TourStepTemplateService, },
    { type: ElementRef, },
    { type: ViewContainerRef, },
    { type: NgZone, },
    { type: TourContentService, },
]; };
BmTourAnchorDirective.propDecorators = {
    'bmTourAnchor': [{ type: Input },],
    'show': [{ type: Output },],
    'hide': [{ type: Output },],
};
