/**
 * Created by gaurav.pandvia on 2/15/17.
 */
import { Input, Component, ElementRef, ViewEncapsulation, ViewChild, HostBinding, Renderer } from '@angular/core';
import { PlacementTypes } from "../utils/placements";
import { PositionHelper } from "../utils/position";
var TourContentComponent = (function () {
    function TourContentComponent(element, renderer) {
        this.element = element;
        this.renderer = renderer;
        this.showCaret = true;
        this.showCursor = false;
        this.alignment = "right";
        this.popoverClass = "";
        this.template = "";
    }
    Object.defineProperty(TourContentComponent.prototype, "cssClasses", {
        get: function () {
            var clz = 'bm-tour-content';
            clz += " position-" + this.placement;
            clz += " " + this.cssClass;
            this.popoverClass = clz;
            return "popover-content";
        },
        enumerable: true,
        configurable: true
    });
    TourContentComponent.prototype.ngAfterViewInit = function () {
        setTimeout(this.position.bind(this));
    };
    TourContentComponent.prototype.position = function () {
        var _this = this;
        // const nativeElm = this.element.nativeElement;
        var nativeElm = this.contentElm.nativeElement;
        this.renderer.setElementClass(nativeElm, 'position-' + this.placement, true);
        this.renderer.setElementClass(nativeElm, this.cssClass, true);
        var highlightedElems = document.querySelectorAll('.highlightOnOverlay');
        for (var i = 0; i < highlightedElems.length; i++) {
            highlightedElems[i].classList.remove('highlightOnOverlay');
            highlightedElems[i].classList.remove('showCursorPointer');
        }
        this.renderer.setElementClass(this.host.nativeElement, 'highlightOnOverlay', true);
        if (this.showCursor) {
            this.renderer.setElementClass(this.host.nativeElement, 'showCursorPointer', true);
        }
        var hostDim = this.host.nativeElement.getBoundingClientRect();
        // if no dims were found, never show
        if (!this.host.nativeElement && !hostDim.height && !hostDim.width)
            return;
        var elmDim = nativeElm.getBoundingClientRect();
        this.checkFlip(hostDim, elmDim);
        this.positionContent(nativeElm, hostDim, elmDim);
        elmDim = nativeElm.getBoundingClientRect(); // Updated nativElm Dimenssions after above function
        if (this.showCaret) {
            this.positionCaret(hostDim, elmDim);
        }
        // animate its entry
        setTimeout(function () { return _this.renderer.setElementClass(nativeElm, 'animate', true); }, 1);
    };
    /**
     *
     * @param nativeElm -- Tour Content Box
     * @param hostDim -- Host Anchor element dimensions
     * @param elmDim --  Tour Content Box dimensions
     */
    TourContentComponent.prototype.positionContent = function (nativeElm, hostDim, elmDim) {
        var _a = PositionHelper.positionContent(this.placement, elmDim, hostDim, this.spacing, this.alignment), top = _a.top, left = _a.left;
        // this.renderer.setElementStyle(nativeElm, 'margin-'+this.oppositePos(this.placement), `15px`);
        this.renderer.setElementStyle(nativeElm, 'top', top + "px");
        this.renderer.setElementStyle(nativeElm, 'left', left + "px");
        //Adjusting position of box, after resizing
        var newElmDim = nativeElm.getBoundingClientRect();
        if ((this.placement === PlacementTypes.left && newElmDim.right > hostDim.left) ||
            this.placement === PlacementTypes.right && newElmDim.left < hostDim.right) {
            this.positionContent(nativeElm, hostDim, newElmDim);
        }
    };
    TourContentComponent.prototype.positionCaret = function (hostDim, elmDim) {
        var caretElm = this.caretElm.nativeElement;
        var caretDimensions = caretElm.getBoundingClientRect();
        var _a = PositionHelper.positionCaret(this.placement, elmDim, hostDim, caretDimensions, this.alignment), top = _a.top, left = _a.left;
        this.renderer.setElementStyle(caretElm, 'top', top + "px");
        this.renderer.setElementStyle(caretElm, 'left', left + "px");
    };
    TourContentComponent.prototype.oppositePos = function (placement) {
        switch (placement) {
            case 'top': return 'bottom';
            case 'bottom': return 'top';
            case 'left': return 'right';
            case 'right': return 'left';
            default: return 'top';
        }
    };
    TourContentComponent.prototype.checkFlip = function (hostDim, elmDim) {
        this.placement = PositionHelper.determinePlacement(this.placement, elmDim, hostDim, this.spacing, this.alignment);
    };
    return TourContentComponent;
}());
export { TourContentComponent };
// @HostListener('window:resize')
// @throttleable(100)
// onWindowResize(): void {
//   this.position();
// }
TourContentComponent.decorators = [
    { type: Component, args: [{
                selector: 'bm-tour-content',
                template: "\n    <div class=\"overlay\"></div>\n    <div class=\"bm-tour-content\"  #content>\n      <span\n        #caretElm\n        [hidden]=\"!showCaret\"\n        class=\"tooltip-caret position-{{this.placement}}\">\n      </span>\n      <div class=\"tooltip-content\">\n      \n          <ng-template\n            [ngTemplateOutlet]=\"template\"\n            [ngOutletContext]=\"{step : context }\">\n          </ng-template>\n       \n      </div>\n    </div>\n  ",
                encapsulation: ViewEncapsulation.None,
                styles: [".highlightOnOverlay {\n  position: relative;\n  z-index: 4999;\n  box-shadow: 0 0 7px 4px rgba(245, 165, 35, 0.34);\n  border-radius: 4px; }\n  .showCursorPointer:before{\n    font-family: FontAwesome;\n    content: \"\\f25a\";\n    position: absolute;\n    color: #3d3d3d;\n    font-size: 18px;\n    font-weight: bolder;\n    bottom: -12px;}\n\n.overlay {\n  background-color: rgba(0, 0, 0, 0.5);\n  position: fixed;\n  top: 0;\n  left: 0;\n  width: 100%;\n  height: 100%;\n  z-index: 1040 !important;\n  content: \"\"; }\n\n.bm-tour-content {\n  position: fixed;\n  max-width: 400px;\n  border-radius: 3px;\n  z-index: 5000;\n  display: block;\n  font-weight: normal;\n  opacity: 0;\n  pointer-events: none;\n  background: #fff;\n  color: #3d3d3d;\n  box-shadow: 0 0 4px 1px #ebeeee;\n  border: 1px solid transparent;\n  font-size: 13px;\n  padding: 15px; }\n  .bm-tour-content .title {\n    font-weight: bold;\n    font-size: 1.8rem;\n    margin-bottom: 10px; }\n  .bm-tour-content .tour-step-content {\n    color: #979797;\n    margin-right: 10px;\n    font-size: 1.4rem; }\n  .bm-tour-content span.close-tour:before {\n    content: 'x';\n    font-weight: bold;\n    font-family: sans-serif;\n    font-size: 15px;\n    color: #979797;\n    line-height: 1;\n    top: 12px;\n    right: 10px;\n    cursor: pointer;\n    position: absolute; }\n  .bm-tour-content span.close-tour:hover::after {\n    font-size: 15px;\n    color: #36b7a4;\n    transition: all 0.2s linear; }\n  .bm-tour-content .btn-color {\n    border-radius: 4px;\n    background-color: #36b7a4;\n    float: right;\n    color: white; }\n  .bm-tour-content .tooltip-caret {\n    position: absolute;\n    z-index: 5001;\n    width: 0;\n    height: 0; }\n    .bm-tour-content .tooltip-caret.position-left {\n      border-top: 7px solid transparent;\n      border-bottom: 7px solid transparent;\n      border-left: 7px solid #fff; }\n    .bm-tour-content .tooltip-caret.position-top {\n      border-left: 7px solid transparent;\n      border-right: 7px solid transparent;\n      border-top: 7px solid #fff; }\n    .bm-tour-content .tooltip-caret.position-right {\n      border-top: 7px solid transparent;\n      border-bottom: 7px solid transparent;\n      border-right: 7px solid #fff; }\n    .bm-tour-content .tooltip-caret.position-bottom {\n      border-left: 7px solid transparent;\n      border-right: 7px solid transparent;\n      border-bottom: 7px solid #fff; }\n  .bm-tour-content .tooltip-caret {\n    position: absolute;\n    z-index: 5001;\n    width: 0;\n    height: 0; }\n  .bm-tour-content.position-right {\n    transform: translate3d(10px, 0, 0); }\n  .bm-tour-content.position-left {\n    transform: translate3d(-10px, 0, 0); }\n  .bm-tour-content.position-top {\n    transform: translate3d(0, -10px, 0); }\n  .bm-tour-content.position-bottom {\n    transform: translate3d(0, 10px, 0); }\n  .bm-tour-content.animate {\n    opacity: 1;\n    transition: opacity 0.3s, transform 0.3s;\n    transform: translate3d(0, 0, 0);\n    pointer-events: auto; }"]
            },] },
];
/** @nocollapse */
TourContentComponent.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: Renderer, },
]; };
TourContentComponent.propDecorators = {
    'host': [{ type: Input },],
    'context': [{ type: Input },],
    'showCaret': [{ type: Input },],
    'showCursor': [{ type: Input },],
    'placement': [{ type: Input },],
    'spacing': [{ type: Input },],
    'cssClass': [{ type: Input },],
    'title': [{ type: Input },],
    'alignment': [{ type: Input },],
    'caretElm': [{ type: ViewChild, args: ['caretElm',] },],
    'contentElm': [{ type: ViewChild, args: ['content',] },],
    'cssClasses': [{ type: HostBinding, args: ['class',] },],
};
