/**
 * Created by gaurav.pandvia on 2/10/17.
 */
import { Injectable } from "@angular/core";
import { Router } from "@angular/router";
import { Subject } from "rxjs/Rx";
import { mergeStatic } from 'rxjs/operator/merge';
import { map } from 'rxjs/operator/map';
export var TourState;
(function (TourState) {
    TourState[TourState["OFF"] = 0] = "OFF";
    TourState[TourState["ON"] = 1] = "ON";
    TourState[TourState["PAUSED"] = 2] = "PAUSED";
})(TourState || (TourState = {}));
var BmTourService = (function () {
    //Anchors place
    function BmTourService(router) {
        this.router = router;
        this.stepShow$ = new Subject();
        this.stepHide$ = new Subject();
        this.initialize$ = new Subject();
        this.initiateStep$ = new Subject();
        this.start$ = new Subject();
        this.end$ = new Subject();
        this.pause$ = new Subject();
        this.resume$ = new Subject();
        this.anchorRegister$ = new Subject();
        this.anchorUnregister$ = new Subject();
        //Flatten Observables
        this.events$ = mergeStatic(map.bind(this.stepShow$)(function (value) { return ({ name: 'stepShow', value: value }); }), map.bind(this.stepHide$)(function (value) { return ({ name: 'stepHide', value: value }); }), map.bind(this.initiateStep$)(function (value) { return ({ name: 'initiateStep', value: value }); }), map.bind(this.initialize$)(function (value) { return ({ name: 'initialize', value: value }); }), map.bind(this.start$)(function (value) { return ({ name: 'start', value: value }); }), map.bind(this.end$)(function (value) { return ({ name: 'end', value: value }); }), map.bind(this.pause$)(function (value) { return ({ name: 'pause', value: value }); }), map.bind(this.resume$)(function (value) { return ({ name: 'resume', value: value }); }), map.bind(this.anchorRegister$)(function (value) { return ({ name: 'anchorRegister', value: value }); }), map.bind(this.anchorUnregister$)(function (value) { return ({ name: 'anchorUnregister', value: value }); }));
        this.anchors = {};
        this.status = TourState.OFF;
    }
    BmTourService.prototype.initialize = function (steps, stepDefaults) {
        if (steps && steps.length > 0) {
            this.steps = steps.map(function (step) { return Object.assign({}, stepDefaults, step); });
            this.initialize$.next(steps);
            this.status = TourState.OFF;
        }
    };
    BmTourService.prototype.start = function () {
        this.startAt(0);
    };
    BmTourService.prototype.startAt = function (stepId) {
        this.goToStep(this.loadStep(stepId));
        this.status = TourState.ON;
        this.start$.next();
        // this.setHotkeys();
    };
    BmTourService.prototype.end = function () {
        this.hideStep(this.currentStep);
        this.currentStep = undefined;
        this.status = TourState.OFF;
        var highlightedElems = document.querySelectorAll('.highlightOnOverlay');
        for (var i = 0; i < highlightedElems.length; i++) {
            highlightedElems[i].classList.remove('highlightOnOverlay');
            highlightedElems[i].classList.remove('showCursorPointer');
        }
        this.end$.next();
        // this.removeHotkeys();
    };
    BmTourService.prototype.pause = function () {
        this.hideStep(this.currentStep);
        this.pause$.next();
        this.status = TourState.PAUSED;
        // this.setHotkeys();
    };
    BmTourService.prototype.resume = function () {
        this.showStep(this.currentStep);
        this.resume$.next();
        this.status = TourState.ON;
        // this.removeHotkeys();
    };
    BmTourService.prototype.toggle = function (pause) {
        if (pause) {
            if (this.currentStep) {
                this.pause();
            }
            else {
                this.resume();
            }
        }
        else {
            if (this.currentStep) {
                this.end();
            }
            else {
                this.start();
            }
        }
    };
    BmTourService.prototype.next = function () {
        this.goToStep(this.loadStep(this.currentStep.nextStep || this.steps.indexOf(this.currentStep) + 1));
    };
    BmTourService.prototype.hasNext = function (step) {
        return step.nextStep !== undefined || this.steps.indexOf(step) < this.steps.length - 1;
    };
    BmTourService.prototype.prev = function () {
        this.goToStep(this.loadStep(this.currentStep.prevStep || this.steps.indexOf(this.currentStep) - 1));
    };
    BmTourService.prototype.hasPrev = function (step) {
        return step.prevStep !== undefined || this.steps.indexOf(step) > 0;
    };
    BmTourService.prototype.goto = function (stepId) {
        this.goToStep(this.loadStep(stepId));
    };
    BmTourService.prototype.register = function (anchorId, anchor) {
        if (this.anchors[anchorId]) {
            return;
        }
        this.anchors[anchorId] = anchor;
        this.anchorRegister$.next(anchorId);
    };
    BmTourService.prototype.unregister = function (anchorId) {
        delete this.anchors[anchorId];
        this.currentStep = undefined;
        this.anchorUnregister$.next(anchorId);
    };
    /**
     * Configures hot keys for controlling the tour with the keyboard
     */
    // private setHotkeys(): void {
    //   this.hotkeyService.add(this.hotkeys);
    // }
    //
    // private removeHotkeys(): void {
    //   this.hotkeyService.remove(this.hotkeys);
    // }
    BmTourService.prototype.goToStep = function (step) {
        var _this = this;
        if (!step) {
            this.end();
            return;
        }
        this.initiateStep$.next(step); //Fire event on next call(Fix for CD)
        if (this.currentStep && this.currentStep.nextStepCallback) {
            this.currentStep.nextStepCallback.call(this.currentStep.callbackRef);
        }
        var navigatePromise = new Promise(function (resolve) { return resolve(true); });
        if (step.route !== undefined && typeof (step.route) === 'string') {
            navigatePromise = this.router.navigateByUrl(step.route);
        }
        else if (step.route && Array.isArray(step.route)) {
            navigatePromise = this.router.navigate(step.route);
        }
        navigatePromise.then(function (navigated) {
            if (navigated !== false) {
                _this.setCurrentStep(step);
            }
        });
    };
    BmTourService.prototype.loadStep = function (stepId) {
        if (typeof (stepId) === 'number') {
            return this.steps[stepId];
        }
        else {
            // return _.find(this.steps,{stepId : stepId});
            return this.steps.find(function (step) { return step.stepId === stepId; });
        }
    };
    BmTourService.prototype.setCurrentStep = function (step) {
        if (this.currentStep) {
            this.hideStep(this.currentStep);
        }
        this.currentStep = step;
        this.showStep(this.currentStep);
    };
    BmTourService.prototype.showStep = function (step) {
        var anchor = this.anchors[step.anchorId];
        if (!anchor) {
            this.end();
            return;
        }
        if (this.status != 0) {
            anchor.showTourStep(step);
        }
        else {
            return;
        }
        // this.stepShow$.next(step);
    };
    BmTourService.prototype.hideStep = function (step) {
        var anchor = this.anchors[step.anchorId];
        if (!anchor) {
            return;
        }
        anchor.hideTourStep();
        this.stepHide$.next(step);
    };
    return BmTourService;
}());
export { BmTourService };
BmTourService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
BmTourService.ctorParameters = function () { return [
    { type: Router, },
]; };
